import { LifeCycle, BaseConfiguration, ExtraConfiguration } from 'miniprogram-network-life-cycle';
import { transfomDownloadSendDefault } from './transform';
import { Omit } from 'miniprogram-network-utils';
/**
 * 默认配置信息
 */
export declare type DownloadInit = BaseConfiguration<DownloadOption, wx.DownloadFileOption>;
/**
 * 全部配置信息
 */
export interface DownloadOption extends DownloadInit, ExtraConfiguration {
    url: NonNullable<string>;
    filePath?: string;
    onProgressUpdate?: wx.DownloadTaskOnProgressUpdateCallback;
}
export declare class Downloder extends LifeCycle<wx.DownloadFileOption, wx.DownloadTask, DownloadInit, DownloadOption> {
    /**
     * 默认下载请求参数转换函数
     */
    protected readonly TransformSendDefault: typeof transfomDownloadSendDefault;
    /**
     * 新建 Http实列
     * @param config 全局默认配置
     */
    constructor(config?: DownloadInit, downloader?: (o: wx.DownloadFileOption) => wx.DownloadTask);
    /**
     * 快速下载
     * @param url 下载地址
     * @param filePath 本地文件路径
     * @param options 其他参数
     */
    download<T = ReturnType<Downloder['TransformResponseDefault']>>(url: string, filePath?: string, options?: Omit<DownloadOption, 'url' | 'filePath'>): Promise<T>;
    /**
     * Object 参数自定义下载
     * @param options
     */
    download<T = ReturnType<Downloder['TransformResponseDefault']>>(options: DownloadOption): Promise<T>;
}
//# sourceMappingURL=downloader.d.ts.map